/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/utils/EventObject.java-arc   1.7   Apr 21 2005 10:51:44   adij  $
 * $Revision:   1.7  $
 * $Date:   Apr 21 2005 10:51:44  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 * Date Changed   Changed By     Comments
 * ------------   ----------     -----------
 * 14-Aug-2012    Bob Jones      7984 Java upgrade from 6 to 7.
 */

package com.capgemini.nemmco.b2bb.utils;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.capgemini.nemmco.b2bb.XMLObject;
import com.capgemini.nemmco.b2bb.enums.EventClass;
import com.capgemini.nemmco.b2bb.enums.EventCode;
import com.capgemini.nemmco.b2bb.enums.Generic;
import com.capgemini.nemmco.b2bb.enums.XMLObjectType;

/**
 * The class <code>EventObject</code> represents an error in a validated xml file.
 *
 * @author $Author:   adij  $
 * @version $Revision:   1.7  $ $Date:   Apr 21 2005 10:51:44  $
 */
public class EventObject extends XMLObject {
  public static String CONTEXT = "context";
  public static String EVENT_CLASS = "eventClass";
  public static String EVENT_CODE = "eventCode";
  public static String EXPLANATION = "explanation";
  public static String KEY_INFO = "keyInfo";
  public static String SUPPORT_VERSIONS = "supportedVersions";
  public static String SEVERITY = "severity";
  public static String EVENT_DESC = "eventDesc";
  
  private String context = "";
  private EventClass eventClass;
  private EventCode eventCode;
  private String eventCodeStr = "";
  private String severity = "";
  private String eventDesc = "";
  private String explanation = "";
  private String keyInfo = "";
  private String supportedVersions = "";

  private final int CONTEXT_MAX_LENGTH = 80;
  private final char XPATH_SEPERATOR = '/';

  /**
   * Constructor to create an EventObject
   * 
   * @param context xpath of the xml object with the error. Refer to 
   * <a href="http://www.w3.org/TR/xpath">http://www.w3.org/TR/xpath</a>
   * for the xpath standard.
   * @param eventCode An event code from which the eventCode, eventDescription and Event Severity 
   * are extracted to populate this object.
   * @param explanation The description of the error.
   */
  public EventObject(String context, EventCode eventCode, String explanation) {
    this();
    // use setContext in order to trim the context
    setContext(context);
    this.eventCode = eventCode;
    this.explanation = explanation;
  }
  
  public EventObject() {
    super(XMLObjectType.EVENT);
  }
  
  public EventObject(XMLObjectType xmlObjectType) {
    super(xmlObjectType);
  }

  /**
   * @return context of the event
   */
  public String getContext() {
    return context;
  }
  
  /**
   * @return Returns the eventClass.
   */
  public String getEventClass() {
    if (eventClass != null) {
      return eventClass.getCode();
    } else {
      return "";
    }
  }

  /**
   * @return event code
   */
  public String getEventCode() {
    if (eventCode != null) {
      return eventCode.getCode();
    } else {
      return eventCodeStr;
    }
  }

  /**
   * @return event description
   */
  public String getEventDesc() {
    if (eventDesc != null && !eventDesc.equals("")) {
      return eventDesc;
    } else if (eventCode != null) {
      return eventCode.getDescription();
    } else {
      return "";
    }
  }

  /**
   * @return error description
   */
  public String getExplanation() {
    return explanation;
  }
  
  /**
   * @return Returns the keyInfo.
   */
  public String getKeyInfo() {
    return keyInfo;
  }

  /**
   * @return severity of the event
   */
  public String getSeverity() {
    if (severity != null && !severity.equals("")) {
      return severity;
    } else if (eventCode != null) {
      return eventCode.getSeverity().getCode();
    } else {
      return "";
    }
  }
  
  /**
   * @return Returns the supportedVersions.
   */
  public String getSupportedVersions() {
    return supportedVersions;
  }
  
  /**
   * @param context The context to set.
   */
  public void setContext(String context) {
    // trim the context to get it to length 80
    this.context = xpathTrimString(context, CONTEXT_MAX_LENGTH);
  }


  /**
   * @param eventClass The eventClass to set.
   */
  public void setEventClass(String eventClass) {
    this.eventClass = EventClass.getEventClass(eventClass);
  }
  
  /**
   * @param eventCode The eventCode to set.
   */
  public void setEventCode(String eventCode) {
    this.eventCode = EventCode.getEventCode(eventCode);
    if (this.eventCode == null) {
      this.eventCodeStr = eventCode;
    }
  }
  
  /**
   * @param eventDesc The eventDesc to set.
   */
  public void setEventDesc(String eventDesc) {
    this.eventDesc = eventDesc;
  }
  
  /**
   * @param explanation The explanation to set.
   */
  public void setExplanation(String explanation) {
    this.explanation = explanation;
  }
  
  /**
   * @param keyInfo The keyInfo to set.
   */
  public void setKeyInfo(String keyInfo) {
    this.keyInfo = keyInfo;
  }
  
  /**
   * @param severity The severity to set.
   */
  public void setSeverity(String severity) {
    this.severity = severity;
  }
  
  /**
   * @param supportedVersions The supportedVersions to set.
   */
  public void setSupportedVersions(String supportedVersions) {
    this.supportedVersions = supportedVersions;
  }
  
  /**
   * @return Returns the event code list 
   */
  public List<Object> getEventCodes() {
    Iterator<? extends Object> iter = EventCode.EVENT_CODES.iterator();
    ArrayList<Object> eventCodes = new ArrayList<Object>();
    Generic displayEventCode;
    EventCode eventCode;
    
    while (iter.hasNext()) {
      eventCode = (EventCode)iter.next();
      displayEventCode = new Generic(eventCode.getCode(), eventCode.getCode() + " - " + eventCode.getDescription() + " - " + eventCode.getSeverity().getCode());
      eventCodes.add(displayEventCode);
    }
    
    return eventCodes;
  }
  
  /**
   * @return Returns the event classes list 
   */
  public List<? extends Object> getEventClasses() {
    return EventClass.EVENT_CLASSES;
  }
  
  /**
   * @param context string to trim
   * @param context_max_length max length context can be
   * @return
   */
  private String xpathTrimString(String context, int context_max_length) {
    if (context == null)
      return null;
    if (context.length() == 0 || context_max_length == 0)
      return "";
    if (context.length() <= context_max_length)
      return context;
    // get the last max length + 1 elements
    String tmpString = context.substring(context.length() - context_max_length - 1);
    // check if the next element is the Xpath seperator character
    if (tmpString.charAt(0) == XPATH_SEPERATOR) {
      // remove the seperator character and return the string which 
      // is now the correct length
      return tmpString.substring(1);
    } else {
      // otherwise remove all characters until the first seperator character
      if (tmpString.indexOf(XPATH_SEPERATOR) < 0)
        return "";
      return tmpString.substring(tmpString.indexOf(XPATH_SEPERATOR));
    }
  }
  
}

