/*
* PlannedInterruptionNotificationBasicValidator.java
*
* The Copyright of this work is vested in the Australian Energy Market
* Operator (AEMO) and the software is issued in confidence for the purpose
* only for which it is supplied. It must not be reproduced in whole or in
* part or used for tendering or manufacturing purposes except under an
* agreement or with the consent in writing of AEMO and then only on the
* condition that this notice is included in any such reproduction.
* No information as to the contents or subject matter of this software or
* any part thereof arising directly or indirectly there from shall be
* given orally or in writing or communicated in any manner whatsoever to
* any third party being an individual firm or company or any employee
* thereof without the prior consent in writing of AEMO.
*
* Copyright Australian Energy Market Operator Limited. 2017
*
* Modification History
*
* Date Changed   Changed By        Description
* ------------   ----------        -----------
* 29-Jun-2017    Harpreet Kaur     Created
* 31-Aug-2017    Harpreet Kaur     7135 Package 7B: Defect#7844 fix
* 17-Oct-2017    Hamish McNeish    8159 B2B Errata make NMI checksum optional
*/

package com.capgemini.nemmco.b2bb.validators;

import com.capgemini.nemmco.b2bb.PlannedInterruptionNotification;
import com.capgemini.nemmco.b2bb.Transaction;
import com.capgemini.nemmco.b2bb.enums.EventCode;
import com.capgemini.nemmco.b2bb.enums.PlannedInterruptionReasons;
import com.capgemini.nemmco.b2bb.utils.B2BDateFormatter;

import java.util.Date;

/**
 * The class <code>PlannedInterruptionNotificationBasicValidator</code> is the Basic Validator
 * object of the Transaction.
 *
 * @author $Author:   HarpreetK  $
 * @version $Revision:   1.0  $ $Date:    30 Jun 2017 11:11:10  $
 */
public class PlannedInterruptionNotificationBasicValidator extends Validator {

   protected PlannedInterruptionNotification transaction;
   private String transactionType = "";

   /**
    * Constructor. Sets a transaction with the objects initialisation.
    * @param transaction Transaction to use in the validator.
    */
   public PlannedInterruptionNotificationBasicValidator(Transaction transaction) {
      super(transaction);
      this.transaction = (PlannedInterruptionNotification) transaction;
      transactionType = transaction.getTransactionType().getCode();
   }

   /**
    * Validate the Transaction
    *
    * @return boolean result of validation
    * validation is called.
    */
   public boolean validate() {
      boolean result = true;

      result &=  validateTransactionID() &
                 validateTransactionDate() &
                 validateNMI() &
                 validateNMIChecksum() &
                 validateStartDate() &
                 validateEndDate() &
                 validateStartTime() &
                 validateDuration() &
                 validateReasonForInterruption() &
                 validateNotes();

      return result;
   }

   /**
    *
    * @return Returns true if startDate is valid
    */
   protected boolean validateStartDate() {
      String fieldname = PlannedInterruptionNotification.START_DATE;
      if (transaction.isStartDateSet()) {

         return validator.validateMandatory(events, category, fieldname, transaction.getStartDate()) &&
                validator.validateXMLDate(events, category, fieldname, transaction.getStartDate());
      } else {
         boolean tempresult = validator.validateMandatory(events, category, fieldname, transaction.getStartDisplayDate());
         tempresult = tempresult && validator.validateB2BDate(events, category, fieldname, transaction.getStartDisplayDate());

         return tempresult;
      }
   }

   /**
    *
    * @return Returns true if endDate is valid
    */
   protected boolean validateEndDate() {
      String fieldname = PlannedInterruptionNotification.END_DATE;

      if (transaction.isEndDateSet()) {
         if (transaction.getEndDate() != null && !transaction.getEndDate().equals("")) {
            try {
               if (transaction.getStartDate() != null && !transaction.getStartDate().equals("")) {
                  B2BDateFormatter df = new B2BDateFormatter(transaction.getEndDate(), B2BDateFormatter.XMLDATE);

                  Date endDate = df.getDate();
                  df = new B2BDateFormatter(transaction.getStartDate(), B2BDateFormatter.XMLDATE);
                  Date startDate = df.getDate();
                  if (startDate.after(endDate)) {
                     Object[] args = { "End Date" };
                     validator.addEvent(events, transactionType, fieldname, args, EventCode.CODE_0202, "invalidEndDateRange");
                     return false;
                  }
               }
            } catch (Exception e) {}
            return validator.validateXMLDate(events, category, fieldname, transaction.getEndDate());
         }
         return true;
      } else {

         if (transaction.getEndDisplayDate() != null && !transaction.getEndDisplayDate().equals("")) {
            try {
               if (transaction.getStartDisplayDate() != null && !transaction.getStartDisplayDate().equals("")) {
                  B2BDateFormatter df =
                     new B2BDateFormatter(transaction.getEndDisplayDate(), B2BDateFormatter.B2BDATE);

                  Date endDate = df.getDate();
                  df = new B2BDateFormatter(transaction.getStartDisplayDate(), B2BDateFormatter.B2BDATE);
                  Date startDate = df.getDate();
                  if (startDate.after(endDate)) {
                     Object[] args = { "End Date" };
                     validator.addEvent(events, transactionType, fieldname, args, EventCode.CODE_0202,"invalidEndDateRange");
                     return false;
                  }
               }
            } catch (Exception e) {}

            return validator.validateB2BDate(events, category, fieldname, transaction.getEndDisplayDate());
         }
         return true;
      }
   }

   /**
    * Validate start time
    * @return boolean
    */
   protected boolean validateStartTime() {
      boolean result;
      String fieldname = PlannedInterruptionNotification.START_TIME;
      if(transaction.getIsStartTimeSet())
            result = validator.validateMandatory(events, category, fieldname, transaction.getStartTime()) &&
                     validator.validateXMLTime(events, category, fieldname, transaction.getStartTime());
         else
            result = validator.validateMandatory(events, category, fieldname, transaction.getStartDisplayTime()) &&
                     validator.validateB2BTime(events,category,fieldname, transaction.getStartDisplayTime());

      return result;
   }

   /**
    * Validate duration
    * @return boolean
    */
   protected boolean validateDuration() {
      boolean result;
      String fieldname = PlannedInterruptionNotification.DURATION;
      String duration = transaction.getDuration();

      result = validator.validateMandatory(events, category, fieldname, duration) &&
               validator.validateLengthRange(events, category, fieldname, duration, 0, 5) &&
               validator.validateDurationInterval(events, category, fieldname, duration);

      return result;
   }

   protected boolean validateReasonForInterruption() {
      String field = PlannedInterruptionNotification.PLANNED_INTERRUPTION_REASON;
      //Optional
      return !validator.exists(transaction.getPlannedInterruptionReason()) || validator.validateEnumeratedValue(events, category, field, "Reason", PlannedInterruptionReasons.PLANNED_INTERRUPTION_REASONS, transaction.getPlannedInterruptionReason());
   }

   protected boolean validateNotes() {
      String fieldname = PlannedInterruptionNotification.NOTES;
      String value = transaction.getNotes();
      String conditionalFieldname = PlannedInterruptionNotification.PLANNED_INTERRUPTION_REASON;
      String conditionalValue = transaction.getPlannedInterruptionReason();
	  //Mandatory when Reason is Other
      return validator.validateMandatoryConditionally1(events, category, fieldname, value, conditionalFieldname, conditionalValue, PlannedInterruptionReasons.OTHER.getCode())
        && validator.validateLengthRange(events, category, fieldname, value, 0, 240);
   }
}
